"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useIsCollapsed = exports.useItemHighlightContext = exports.useItemContext = exports.itemContext = void 0;
const react_1 = require("react");
const numbers_1 = require("../utils/numbers");
const treeToRenderAdapter_1 = require("./treeToRenderAdapter");
const initialState = {
    renderData: [],
    hoveredItems: [],
    collapsedItems: [],
};
const store = react_1.createContext({
    state: {
        renderData: [[]],
        hoveredItems: [],
        collapsedItems: [],
    },
    dispatch: () => { },
});
exports.itemContext = store;
const useItemContext = (tree) => {
    const { newResult: renderDataRaw } = treeToRenderAdapter_1.generateRenderTree(tree);
    const [, ...renderMatrix] = renderDataRaw;
    return react_1.useReducer((previousState, action) => {
        var _a, _b;
        switch (action.type) {
            case 'TOGGLE_COLLAPSE': {
                const { currentItem } = (_a = action.payload) !== null && _a !== void 0 ? _a : {};
                const { path } = currentItem;
                const indexOFRemovedItem = previousState.collapsedItems.findIndex(itemPath => numbers_1.numArrayComparison(itemPath, path));
                const newCollapsedItems = indexOFRemovedItem === -1
                    ? [...previousState.collapsedItems, path]
                    : previousState.collapsedItems.filter((item, index) => index !== indexOFRemovedItem);
                const { newResult: newRenderDataRaw } = treeToRenderAdapter_1.generateRenderTree(tree, newCollapsedItems);
                const [, ...newRenderData] = newRenderDataRaw;
                return Object.assign(Object.assign({}, previousState), { collapsedItems: newCollapsedItems, renderData: newRenderData });
            }
            case 'SET_HOVERED': {
                const { currentHoveredItem, hoveredColumnIndex: columnIndex, } = (_b = action.payload) !== null && _b !== void 0 ? _b : {};
                const path = currentHoveredItem
                    ? findPathFromItem([currentHoveredItem.id], currentHoveredItem.parentId, previousState.renderData.slice(0, Math.max(columnIndex, 0)))
                    : [];
                return Object.assign(Object.assign({}, previousState), { hoveredItems: path });
            }
            case 'SET_BLURRED': {
                return Object.assign(Object.assign({}, previousState), { hoveredItems: [] });
            }
            default:
                throw new Error();
        }
    }, Object.assign(Object.assign({}, initialState), { renderData: renderMatrix }));
};
exports.useItemContext = useItemContext;
const useItemHighlightContext = ({ currentItem, }) => {
    const { state: { hoveredItems }, } = react_1.useContext(exports.itemContext);
    const highlighted = hoveredItems.some(itemId => itemId === currentItem.id);
    return highlighted;
};
exports.useItemHighlightContext = useItemHighlightContext;
const useIsCollapsed = ({ currentItem, }) => {
    const { state: { collapsedItems }, } = react_1.useContext(exports.itemContext);
    const isCollapsed = collapsedItems.some(collapsedPath => numbers_1.numArrayComparison(collapsedPath, currentItem.path));
    return isCollapsed;
};
exports.useIsCollapsed = useIsCollapsed;
function findPathFromItem(result = [], parentId, renderMatrix) {
    if (renderMatrix.length === 0) {
        return result;
    }
    if (renderMatrix.length === 1) {
        return [parentId, ...result];
    }
    const currentParentItem = renderMatrix[renderMatrix.length - 1].find(parent => parent.id === parentId);
    const newRenderMatrix = renderMatrix.slice(0, -1);
    return findPathFromItem([currentParentItem.id, ...result], currentParentItem.parentId, newRenderMatrix);
}
//# sourceMappingURL=item-context.js.map